-- Pepe Collection 1.6.3 by Hrazmadul
-- Updated 2022-06-01
-- 1.6.3 Updated TOC for 9.2.5
-- 1.6.2 Updated TOC for 9.2
-- 1.6.1 Updated TOC for 9.1.5
-- 1.6.0 Changed reading of the players covenant
-- 1.5.9 Added Renown 56 Pepes
-- 1.5.8 Updated TOC for 9.1.0
-- 1.5.7 Updated TOC for 9.0.5
-- 1.5.6 Updated the Winter Pepe Coords
-- 1.5.5 Updated TOC for 9.0.2 and shortened Pin text a bit.
-- 1.5.4 Added the new Map Pins from 9.0.1 to some pepes, you can only have one pin active.
-- 1.5.3 Updated TOC to Shadowlands Prepatch
-- 1.5.2 Separated horde and alliance waypoints for clearer output.
-- 1.5.1 Updated to handle list of coordinates for Traveller Pepe.
local addonName
addonName = ...

local version="1.6.3"
local updated="2022-06-01"

local f = CreateFrame("Frame");

local col_heirloom	= "cff00ccff"
local col_artifact	= "cffe6cc80"
local col_legendary	= "cffff8000"
local col_epic		= "cffa335ee"
local col_rare		= "cff0070dd"
local col_uncommon	= "cff1eff00"
local col_common		= "cffffffff"
local col_poor		= "cff9d9d9d"

local col_mogit		= "cffcc99ff"
local col_pepe		= col_mogit
local col_zonetext	= "cff8080ff"	-- cff8080ff purple
local col_title		= "cffffb200"	-- cffffb200 orange
local col_b			= "|"
local col_e			= "|r"

local playerCovenantID = 0

local lastAddedWaypoint = nil;
local function addTomTomWaypoint(map, zone, coordX, coordY, text)
	if (TomTom) then
		local mapzonename = map
		local slask, mapid, coordlist
	
		if (map:find("LIST")) then -- coord list
			slask, mapid = map:match("(%d*);(%d*)");
			coordlist = { strsplit("/", zone) }
			for i=1,table.getn(coordlist) do
				local tempx, tempy = coordlist[i]:match("([^:]*),([^:]*)");

				lastAddedWaypoint = TomTom:AddWaypoint(tonumber(mapid), tonumber(tempx)/100, tonumber(tempy)/100, {
					title = "Pepe Collection - " .. date("%H:%M").. " - " .. text,
					from = "Pepe Collection",
					persistent = nil,
					minimap = true,
					world = true
				})
			end
			print("Added to TomTom : Coord List (Map:"..mapid..") for "..text)
		elseif  (map:find("MAP")) then -- special map coord
			slask, mapid = map:match("(%d*);(%d*)");
			
			lastAddedWaypoint = TomTom:AddWaypoint(tonumber(mapid), tonumber(coordX)/100, tonumber(coordY)/100, {
				title = "Pepe Collection - " .. date("%H:%M").. " - " .. text,
				from = "Pepe Collection",
				persistent = nil,
				minimap = true,
				world = true
			})
			print("Added to TomTom : Map:"..mapid.." "..zone.." "..coordX.." "..coordY.." "..text)
		elseif (map=="MZCW") then -- special map zone continent world coord
			local idmap, idzone, idcontinent, idworld = zone:match("(%d*)/(%d*)/(%d*)/(%d*)");
			mapzonename = "M:"..idmap.." Z:"..idzone.." C:"..idcontinent.." W:"..idworld
			print("Added to TomTom : "..mapzonename.." "..coordX.." "..coordY.." "..text)

			lastAddedWaypoint = TomTom:AddWaypoint(tonumber(idmap), tonumber(coordX)/100, tonumber(coordY)/100, {
				title = "Pepe Collection - " .. date("%H:%M").. " - " .. text,
				from = "Pepe Collection",
				persistent = nil,
				minimap = true,
				world = true
			})
		else -- ordinary coord
			if not (zone=="") then
				mapzonename = mapzonename..":"..zone
			end
			print("Added to TomTom : "..mapzonename.." "..coordX.." "..coordY.." "..text)
			lastAddedWaypoint = TomTom:AddWaypoint(TomTom.NameToMapId[mapzonename], tonumber(coordX)/100, tonumber(coordY)/100, {
				title = "Pepe Collection - " .. date("%H:%M").. " - " .. text,
				from = "Pepe Collection",
				persistent = nil,
				minimap = true,
				world = true
			})
		end
	else -- no tomtom
		print "TomTom not loaded!"
	end
end

local function LinkToVar(link)
	local set = {};
	-- local items, race, gender, enchant = strsplit(":", link:match("MogIt:(.+)"));
    local map, zone, cx, cy, tttext = link:match("TOMTOM:([^:]*):([^:]*):([^:]*):([^:]*):([^|]*)");
	return map, zone, cx, cy, tttext;
end

local function filter(self, event, msg, ...)
	msg = msg:gsub("%[(TOMTOM[^%]]+)%]","|"..col_pepe.."|H%1|h[TomTom]|h|r");
	return false, msg, ...;
end
ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", filter);

local SetHyperlink = ItemRefTooltip.SetHyperlink;
function ItemRefTooltip:SetHyperlink(link)
  if link:find("^TOMTOM") then
    if IsModifiedClick("CHATLINK") then
      ChatEdit_InsertLink("["..link.."]")
    else
      local map, zone, cx, cy, tttext = LinkToVar(link);

      if (TomTom) then
        addTomTomWaypoint(map, zone, cx, cy, tttext)
      else
        -- StaticPopup_Show("CLICK_LINK_CLICKURL", "", "", d)
      end
    end
    else
      SetHyperlink(self, link);
    end
end

StaticPopupDialogs["CLICK_LINK_CLICKURL"] = {
    text = "Copy & Paste the tomtom waypoint",
    button1 = "Close",
    OnAccept = function()
    end,
    timeout = 0,
    whileDead = true,
    hideOnEscape = true,
    preferredIndex = 3, 
    OnShow = 
        function (self, data)
            self.editBox:SetText(data.tomtom)
            self.editBox:HighlightText()
        end,
    hasEditBox = true
}

-------------------------------------------------------------------
-- Command line processing
-------------------------------------------------------------------
-- traveller pepe coords
--TomTom: You are at (56.85, 50.61) in 'Dalaran' (map: 627)
--TomTom: Dalaran: (map: 627, zone: 627, continent: 619, world: 947)
--TomTom:    TomTom waypoint - 49.97, 33.76 627 * *
--49.12,32.40/39.15,33.36/39.25,25.10/51.27,53.49/57.12,53.05/36.84,37.06/48.65,40.20/32.25,31.66/55.39,44.14/50.59,57.19/36.17,27.70/57.87,42.28/59.87,52.53/59.87,52.53/41.88,33.03

-- Name						ItemID	Quest	X1		Y1		X2		Y2
-- A Tiny Viking Helmet		127865	39265	80.18	50.36	TomTom: You are at (80.18, 50.36) in 'Nagrand' (map: 550)
-- A Tiny Ninja Shroud		127867	39267	47.50	41.30	TomTom: You are at (47.50, 41.30) in 'Gorgrond' (map: 543)
-- A Tiny Plated Helm		127869	39266	50.97	63.30	TomTom: You are at (50.97, 63.30) in 'Talador' (map: 535)
-- A Tiny Pirate Hat		127870	39268	54.12	83.61	TomTom: You are at (54.12, 83.61) in 'Spires of Arak' (map: 542)
-- A Tiny Scarecrow Costume	128874	39865	H50.89	H70.40	A29.40	A19.70  problems setting a waypoint with tomtom
-- A Tiny Pair of Goggles	139632	43695	*	*	too many points
-- A Tiny Voodoo Mask		161443	52269	39.04	16.04	TomTom: You are at (39.04, 16.04) in 'Dazar'alor' (map: 1165)
-- A Tiny Diving Helmet		161451	52277	54.40	72.61	TomTom: You are at (54.40, 72.61) in 'Boralus' (map: 1161)
-- A Tiny Clockwork Key		170151	56911	68.47	47.85	TomTom: You are at (68.47, 47.85) in 'Mechagon Island' (map: 1462)
-- A Tiny Winter Hat		174865	58901	H52.9	H77.5	A34.3	A68.5	org(map 85)/if(map 87)
-- Winter Pepe TomTom: You are at (33.95, 68.21) in 'Ironforge' (map: 87)
-- Winter Pepe TomTom: You are at (52.98, 77.43) in 'Orgrimmar' (map: 85)
-- TomTom: You are at (---) in 'Unnamed Map' (map: nil)
-- TomTom: Lunarfall: (map: 582, zone: 539, continent: 572, world: 572)
-- TomTom: Frostwall: (map: 590, zone: 525, continent: 572, world: 572)
-- Name						ItemID	Quest	X1		Y1		X2		Y2
-- A Tiny Winter Staff		186473	64078					https://www.wowhead.com/item=186473/a-tiny-winter-staff
-- A Tiny Vial of Slime		186524	64098					https://www.wowhead.com/item=186524/a-tiny-vial-of-slime
-- A Tiny Sinstone			186580	64132					https://www.wowhead.com/item=186580/a-tiny-sinstone
-- A Tiny Pair of Wings		186593	64136					https://www.wowhead.com/item=186593/a-tiny-pair-of-wings

local function ItemLink(color,id,idtext)
  linktext = "|"..color.."|Hitem:"..id.."::::::::120:::::|h["..idtext.."]|h|r"
return linktext
end

local function TomTomLink(map, zone, x, y, text)
  local tomtomstring = ""
  local mapzone = map
  if not (zone=="") then
     mapzone = mapzone..","..zone
  end
  if (TomTom) then
    tomtomstring = format("|%s|HTOMTOM:%s:%s:%s:%s:%s|h[TomTom]|h|r",col_pepe, map, zone, x, y, text)
  else
  	if (map:find("LIST")) then
		tomtomstring = "Too many points to link all"
  	elseif (map:find("MAP")) then
	    local slask, mapid = map:match("(%d*);(%d*)");
		tomtomstring = format("Map:%s %s (%s,%s) %s", mapid, zone, x, y, text)
	else
		tomtomstring = format("%s (%s,%s) %s", mapzone, x, y, text)
	end
  end
 return tomtomstring
end

  -- https://wow.gamepedia.com/API_C_Map.SetUserWaypoint
  -- C_Map.SetUserWaypoint(point)
  -- uiMapPoint = UiMapPoint.CreateFromCoordinates(mapID, x, y, z)
  -- uiMapPoint = UiMapPoint.CreateFromVector2D(mapID, position, z)
  -- uiMapPoint = UiMapPoint.CreateFromVector3D(mapID, position)
  -- local uiMapPoint = { uiMapID = mapID, position = CreateVector2D(x, y), z = z };
  -- 1161 map 51,64
  -- C_Map.SetUserWaypoint(UiMapPoint.CreateFromCoordinates(1161, tonumber(51)/100, tonumber(64)/100, tonumber(0)/100))
  -- below work to set a map pin
  -- C_Map.SetUserWaypoint(UiMapPoint.CreateFromCoordinates(1161, tonumber(51)/100, tonumber(64)/100, tonumber(0)/100))
  -- "|cffffff00|Hworldmap:84:7222:2550|h[|A:Waypoint-MapPin-ChatIcon:13:13:0:0|a Map Pin Location]|h|r"
  -- print(MapPinLink(1161,51,64,"mappy"))

local function MapPinLink(mapid,x,y, pintext)
  local coordx=tonumber(x)*100
  local coordy=tonumber(y)*100
  return format("|cffffff00|Hworldmap:%s:%s:%s|h[|A:Waypoint-MapPin-ChatIcon:13:13:0:0|a %s]|h|r", mapid, coordx, coordy, pintext)
end

local function factionHalloween()
	local fac
	local englishFaction, localizedFaction = UnitFactionGroup("player")
	--" (H)"..TomTomLink("MAP;590","590","70.7","90.1","Scarecrow Pepe").." (A)"..TomTomLink("MAP;582","582","40.08","69.60","Scarecrow Pepe")},
	if (englishFaction=="Horde") then
		fac="(H)"..TomTomLink("MAP;590","Frostwall","70.7","90.1","Scarecrow Pepe") -- ..MapPinLink(590,70.7,90.1,"Scarecrow Pepe") -- doesnt work in garr
	elseif (englishFaction=="Alliance") then
		fac="(A)"..TomTomLink("MAP;582","Lunarfall","40.08","69.60","Scarecrow Pepe") -- ..MapPinLink(582,40.08,69.60,"Scarecrow Pepe") -- doesnt work in garr
	end
	return fac
end

local function factionWinter()
	local fac
	local englishFaction, localizedFaction = UnitFactionGroup("player")
	--" (H)"..TomTomLink("Orgrimmar","","52.9","77.5","Greatfather Winter Pepe").." (A)"..TomTomLink("Ironforge","","34.3","68.5","Greatfather Winter Pepe")
	if (englishFaction=="Horde") then
		fac="(H)"..TomTomLink("Orgrimmar","","52.98","77.43","Greatfather Winter Pepe")..MapPinLink(85, 52.98, 77.43, "Greatfather")
	elseif (englishFaction=="Alliance") then
		fac="(A)"..TomTomLink("Ironforge","","33.95","68.21","Greatfather Winter Pepe")..MapPinLink(87, 33.95, 68.21,"Greatfather")
	end
	return fac
end

-- https://wowpedia.fandom.com/wiki/Category:API_namespaces/C_Covenants
-- 0	None
-- 1	Kyrian
-- 2	Venthyr
-- 3	NightFae
-- 4	Necrolord
-- https://wowpedia.fandom.com/wiki/API_C_Covenants.GetActiveCovenantID
-- some issues info not available until reload or similar
local function covenantUser()
	local covenantID
	covenantID = C_Covenants.GetActiveCovenantID()
	return covenantID
end

local function covenantText(checkcov)
    local covt
	if (checkcov==0) then
	  covt="None"
	elseif (checkcov==1) then
	  covt="Kyrian"
	elseif (checkcov==2) then
	  covt="Venthyr"
	elseif (checkcov==3) then
	  covt="NightFae"	
	elseif (checkcov==4) then
	  covt="Necrolord"
	end
	return covt
end

-- nagrand draenor 550 gorgrond ?? talador ?? spires 542 dazalor ?? boralus ?? mecha
-- kept for reference
--{"A Tiny Scarecrow Costume",	"L3 Garrison Halloween",		39865,	"128874",	col_uncommon,	" (H)"..TomTomLink("Frostfire Ridge","","50.89","70.40","Scarecrow Pepe").." (A)"..TomTomLink("Shadowmoon Valley","Draenor","29.40","19.70","Scarecrow Pepe")},
--{"A Tiny Scarecrow Costume",	"L3 Garrison Halloween",		39865,	"128874",	col_uncommon,	" (H)"..TomTomLink("MZCW","590/525/572/572","70.7","90.1","Scarecrow Pepe").." (A)"..TomTomLink("MZCW","582/539/572/572","40.08","69.60","Scarecrow Pepe")},
--{"A Tiny Scarecrow Costume",	"L3 Garrison Halloween",		39865,	"128874",	col_uncommon,	" (H)"..TomTomLink("MAP;590","590","70.7","90.1","Scarecrow Pepe").." (A)"..TomTomLink("MAP;582","582","40.08","69.60","Scarecrow Pepe")},
--{"A Tiny Pair of Goggles",	"Legion Dalaran",				43695,	"139632",	col_uncommon,	" Too many points to link all"},
--{"A Tiny Winter Hat",			"Winters Veil Vendor Wagon",	58901,	"174865",	col_uncommon,	" (H)"..TomTomLink("Orgrimmar","","52.9","77.5","Greatfather Winter Pepe").." (A)"..TomTomLink("Ironforge","","34.3","68.5","Greatfather Winter Pepe")}

-- A Tiny Winter Staff		186473	64078					https://www.wowhead.com/item=186473/a-tiny-winter-staff
-- A Tiny Vial of Slime		186524	64098					https://www.wowhead.com/item=186524/a-tiny-vial-of-slime
-- A Tiny Sinstone			186580	64132					https://www.wowhead.com/item=186580/a-tiny-sinstone
-- A Tiny Pair of Wings		186593	64136					https://www.wowhead.com/item=186593/a-tiny-pair-of-wings


local quests = {
-- Text							Location						Quest	ItemID		Color			Waypoint
{"A Tiny Viking Helmet",		"Draenor, Nagrand",				39265,	"127865",	col_uncommon,	" "..TomTomLink("Nagrand","Draenor","80.18","50.36","Viking Pepe")..MapPinLink(550,80.18,50.36,"Viking")},
{"A Tiny Ninja Shroud",			"Draenor, Gorgrond",			39267,	"127867",	col_uncommon,	" "..TomTomLink("Gorgrond","","47.50","41.30","Ninja Pepe")..MapPinLink(543, 47.50, 41.30,"Ninja")},
{"A Tiny Plated Helm",			"Draenor, Talador",				39266,	"127869",	col_uncommon,	" "..TomTomLink("Talador","","50.97","63.30","Knight Pepe")..MapPinLink(535, 50.97, 63.30,"Knight")},
{"A Tiny Pirate Hat",			"Draenor, Spires of Arak",		39268,	"127870",	col_uncommon,	" "..TomTomLink("Spires of Arak","","54.12","83.61","Pirate Pepe")..MapPinLink(542, 54.12, 83.61,"Pirate")},
{"A Tiny Scarecrow Costume",	"L3 Garrison Halloween",		39865,	"128874",	col_uncommon,	" "..factionHalloween()},
{"A Tiny Pair of Goggles",		"Legion Dalaran",				43695,	"139632",	col_uncommon,	" "..TomTomLink("LIST;627","49.12,32.40/39.15,33.36/39.25,25.10/51.27,53.49/57.12,53.05/36.84,37.06/48.65,40.20/32.25,31.66/55.39,44.14/50.59,57.19/36.17,27.70/57.87,42.28/59.87,52.53/59.87,52.53/41.88,33.03","","","Traveller Pepe")},
{"A Tiny Voodoo Mask",			"BfA Dazar'alor",				52269,	"161443",	col_uncommon,	" "..TomTomLink("Dazar'alor","Zuldazar","39.04","16.04","Pepe'jin")..MapPinLink(1165, 39.04, 16.04,"Pepe'jin")},
{"A Tiny Diving Helmet",		"BfA Boralus",					52277,	"161451",	col_uncommon,	" "..TomTomLink("Boralus","","54.40","72.61","Diver Pepe")..MapPinLink(1161, 54.40, 72.61,"Diver")},
{"A Tiny Clockwork Key",		"BfA Alternate Mechagon",		56911,	"170151",	col_uncommon,	" "..TomTomLink("Mechagon Island","","68.47","47.85","Clockwork Pepe")..MapPinLink(1462, 68.47, 47.85,"Clockwork")},
{"A Tiny Winter Hat",			"Winters Veil Vendor Wagon",	58901,	"174865",	col_uncommon,	" "..factionWinter()},
{"A Tiny Winter Staff",			"Shadowlands Renown 56",		64078,	"186473",	col_uncommon,	""},
{"A Tiny Vial of Slime",		"Shadowlands Renown 56",		64098,	"186524",	col_uncommon,	""},
{"A Tiny Sinstone",				"Shadowlands Renown 56",		64132,	"186580",	col_uncommon,	""},
{"A Tiny Pair of Wings",		"Shadowlands Renown 56",		64136,	"186593",	col_uncommon,	""}
}

--             Text                             	Location		ID		Color
local toys = {{"Trans-Dimensional Bird Whistle",	"Draenor Achivement",	"122293",	col_epic},
              {"Crate of Bobbers: Wooden Pepe",		"Legion Fishing",		"143662",	col_rare},
              {"A Tiny Set of Warglaives",			"Legion DH Classhall",	"147537",	col_epic}}

SLASH_PEPE1 = "/pepe"
SlashCmdList["PEPE"] = function(args)
  local cmd, arg = strsplit(" ",args,2)
  local qdone, qname, tname, tdone, tomtext
  cmd = cmd:lower()

  print("Pepe Collection "..version.." by Hrazmadul "..updated)

  print(col_b..col_title.."Toys"..col_e)
  for i=1,table.getn(toys) do
    tdone=PlayerHasToy(toys[i][3])
                                                                                --  color      id         text         location
    print(format("%s ("..col_b..col_zonetext.."%s"..col_e.."): %s",ItemLink(toys[i][4],toys[i][3],toys[i][1]), toys[i][2], tdone and "\124cff00ff00Yes\124r" or "\124cffff0000No\124r"))
  end
  print(col_b..col_title.."Quests"..col_e)
  for i=1,table.getn(quests) do
    qdone=C_QuestLog.IsQuestFlaggedCompleted(quests[i][3])

--  if not qdone then
--    tomtext=quests[i][6]
--  else
--    tomtext=""
--  end
  tomtext = quests[i][6] -- always print out the tomtom link
  --special covenant test
  if ((quests[i][3]==64078) or (quests[i][3]==64098) or (quests[i][3]==64132) or (quests[i][3]==64136)) then

    if (quests[i][3]==64078) then --nightfae
	  if (playerCovenantID==3) then tomtext=" You are "..covenantText(3) end
	elseif (quests[i][3]==64098) then --necrolord
	  if (playerCovenantID==4) then tomtext=" You are "..covenantText(4) end
	elseif  (quests[i][3]==64132) then --venthyr
	  if (playerCovenantID==2) then tomtext=" You are "..covenantText(2) end
	elseif  (quests[i][3]==64136) then --kyrian
	  if (playerCovenantID==1) then tomtext=" You are "..covenantText(1) end
	end
  end
                                                                                 -- color        id           text           location
    print(format("%s ("..col_b..col_zonetext.."%s"..col_e.."): %s%s",ItemLink(quests[i][5],quests[i][4],quests[i][1]), quests[i][2], qdone and "\124cff00ff00Yes\124r" or "\124cffff0000No\124r", tomtext))
  end
end

function Pepe_OnEvent(frame, event, name, ...)
  -- had some issues getting the covenant it yielded 0 often
  playerCovenantID = covenantUser()
  -- print("covenant id="..playerCovenantID)
end

f:SetScript("OnEvent", Pepe_OnEvent);
-- testing changing from addon loaded to player login
f:RegisterEvent("ADDON_LOADED")
--f:RegisterEvent("PLAYER_LOGIN")
